include sort.e
include misc.e
--puts(1,"math.e Don Cahela 2004\n")
--MATH.E v1.3 by Andy Montgomery
--Modified by Don Cahela, June 21, 2004.
--Added complex functions July 10, 2004.
--Modifications are to use the constant (PI) and functions already provided
--in "misc.e" and "sort.e".  Also, removed unecessary complications in exp(), sinh(), cosh(), etc.
--The test code demonstrates that power(e,x) is about 225 times faster then
--summing the series expansion for the exponential.
--November 10, 2005.  Added error function and complementary error function.
--See readme.txt for more release history.

--Mathematical routines that work on atoms or seqences.
--global functions:
--lin_intp(atom x1, atom x2, atom y1, atom y2, atom y)
--abs(object x) - absolute value
--exp(object x) - exponential = power(2.71828182845904523536,x)
--kdelta(integer i, integer j) - knoneker delta function.
--sign(object x) - returns -1 for negative numbers, 0 for 0, and 1 for positives.
--cosh(object x) - hyperbolic cosine
--sinh(object x) - hyperbolic sine
--tanh(object x) - hyperbolic tangent
--erf(object x) -  error function
--effc(object x) - complementary error function

--logbase(atom y, object x)- logarithm base y of x
--log10(object x) - common logarithm (base 10)

--oned(object x) - converts atoms or multidimentional sequences to 1D sequences
--fib(object x) - returns the nth Fibonanci number
--factorial(object x) - factorial
--nPr(object a, object b) - number of permutations
--nCr(object a, object b) - number of combinations

--global function determinate_eval(integer n, sequence a)

--Some statistical functions:
--max(object x) - highest value of a sequence (also accepts atoms, but why?)
--min(object x) - lowest value of a sequence (ditto)
--mean(object x) - average of a sequence, averages nested sequences first for weighting
--median(object x) - median of a sequence, middle value of sorted sequence
--variance(object x) - variance of a sequence
--std_dev(object x) - standard deviation of a sequence
--studentT95(integer n) - Regression of student t distribution for 95 % confidence interval,
--that is a onesided tail of 0.025.  This fuction takes a positive integer.

--Note by A.M.-Check out the formula used in fib().  I was flabbergasted when my math
--teacher showed it to us.  It's not recursive!

--Constants:
--nan is the "not a number" the Euphoria documentation talks about, inf
--is the "infinity" it mentions.

global constant 
    nan=-(2*power(10,308)/(3*power(10,308)))--, 
    --inf=2*power(10,308),
    --infsmall=power(10,-308),
    --sqrtmaxatom=power(10,154),
    --sqrtminatom=power(10,-154)

global function lin_intp(atom x1, atom x2, atom y1, atom y2, atom y)
    return x1+(x2-x1)*(y-y1)/(y2-y1)
end function

global function abs(object x)
object y
if sequence(x) then
    y = repeat(0,length(x))
    for count=1 to length(x) do
	y[count] = abs(x[count])
    end for
else
    if x<0 then
	y = -x
    else
	y = x
    end if
end if
return y
end function

global function exp(object x)
object y
    y = power(2.71828182845904523536,x)
return y
end function

--Error functions
function erfcxmgt1(atom x)
atom temp, y, x2
integer count
--Asymptotic series for complementary error function for x>>1:
    x2=power(x,2)
    temp=1
    y=1
    count=-1
--Each term adds about one-third digit of precision for x of about 5
    while ((abs(temp)>1e-308)and(count<19)) do
	count+=1
	temp*=-(2*count+1)/2/x2
	y+=temp
    end while
    y*=exp(-x2)/x/sqrt(PI)
    return y
end function

global atom xerrcut
xerrcut=4

function erfatom(atom x)
atom y, temp, err, x2, absx
integer count
    absx=abs(x)
    x2=power(x,2)
    err=1
    count=1
    if absx<xerrcut then
--Maclaurin series      
	temp=-power(absx,3)
	y=absx+temp/3
	while (abs(err)>1e-308) do
	    count+=1
	    temp*=-x2/count
	    err=temp/(2*count+1)
	    y+=err
	end while
	y*=2/sqrt(PI)
--Error function is an odd function.
	if x<0  then y=-y end if
    else
--Asymptotic series for x>>1 used for x>5           
	y=-erfcxmgt1(x)
	y+=1
    end if
return y
end function

function erfcatom(atom x)
--Complementery error function=1-erf(x)
atom y
    if abs(x)<xerrcut then
--Maclaurin series      
	y=-erfatom(x)
	y+=1
    else
--Asymptotic series for x>>1 used for x>5           
	y=erfcxmgt1(x)
    end if
return y
end function

global function erf(object x)
object y
if sequence(x) then
    y = repeat(0,length(x))
    for count=1 to length(x) do
	y[count] = erfatom(x[count])
    end for
else
    y=erfatom(x)
end if
return y
end function

global function erfc(object x)
object y
if sequence(x) then
    y = repeat(0,length(x))
    for count=1 to length(x) do
	y[count] = erfcatom(x[count])
    end for
else
    y=erfcatom(x)
end if
return y
end function

global function kdelta(integer i, integer j)
--The kronecker delta function
integer kd
    if i=j then
	kd=1
    else
	kd=0
    end if
    return kd
end function

global function sign(object x)
object y
if sequence(x) then
    y = repeat(0,length(x))
    for count=1 to length(x) do
	y[count] = sign(x[count])
    end for
else
    if x<0 then
	y=-1
    elsif x>0 then
	y=1
    elsif x=0 then
	y=0
    else
	y=nan
    end if
end if
return y
end function

global function cosh(object x)
object y
    y = exp(x)/2 + exp(-x)/2
return y
end function

global function sinh(object x)
object y
    y = exp(x)/2 - exp(-x)/2
return y
end function

global function tanh(object x)
object y
    y = (exp(2*x) - 1)/(exp(2*x) + 1)
return y
end function

global function logbase(atom y, object x)
object z
if sequence(x) then
    z = repeat(0,length(x))
    for count=1 to length(x) do
	y[count] = logbase(y,x[count])
    end for
else
    if (x>0 and y>0) then
	z = log(x)/log(y)
    else
	z = nan
    end if
end if
return z
end function

global function log10(object x)
return logbase(10,x)
end function

--Some statistical functions
global function max(object x)
atom y
if sequence(x) then
    y = x[1]
    for count=2 to length(x) do
	if atom(x[count]) then
	if x[count]>y then
	    y = x[count]
	end if
	end if
    end for
else
    y = x
end if
return y
end function

global function min(object x)
atom y
if sequence(x) then
    y = x[1]
    for count=2 to length(x) do
	if atom(x[count]) then
	if x[count]<y then
	    y = x[count]
	end if
	end if
    end for
else
    y = x
end if
return y
end function

global function mean(object x)
atom y
y = 0
if sequence(x) then
    for count=1 to length(x) do
	x[count] = mean(x[count])
	y = y + x[count]
    end for
    y = y/length(x)
else
    y = x
end if
return y
end function

global function oned(object x)
sequence y
if sequence(x) then
    y = {}
    for count=1 to length(x) do
	y = y & oned(x[count])
    end for
else
    y = {x}
end if
return y
end function

global function median(object x)
atom y
if sequence(x) then
    x = sort(x)
    if integer(length(x)/2) then
	y = (x[length(x)/2] + x[length(x)/2+1])/2
    else
	y = x[length(x)/2+.5]
    end if
else
    y = x
end if
return y
end function

--Original function was N variance.
global function variance(object x)
atom y, average, average2
integer n
n=length(x)
average2 = mean(power(x,2))
average = mean(x)
y = average2 - average * average

--Convert from n to n-1 variance
y = n/(n-1)*y
return y
end function

global function std_dev(object x)
atom y, var
var = variance(x)
y = sqrt(var)
return y
end function

global function fib(object x)
--c_gr is the golden ratio, (1+sqrt(5))/2, c_grneg is
--a number related to the golden ratio that is used in the fib() formula,
object y
atom c_gr, c_grneg
c_gr=1.6180339887499
c_grneg=-0.6180339887499
if sequence(x) then
    y=repeat(0,length(x))
    for count=1 to length(x) do
	y[count]=fib(x[count])
    end for
elsif (integer(x) and x>=0) then
    y=floor((power(c_gr,x)-power(c_grneg,x))/sqrt(5))
else
    y=nan
end if
return y
end function

global function factorial(object x)
object y
if sequence(x) then
    y=repeat(0,length(x))
    for count=1 to length(x) do
	y[count]=factorial(x[count])
    end for
elsif integer(x) then
    if x>0 then
	y=x*factorial(x-1)
    elsif x=0 then
	y=1
    else
	y=nan
    end if
else
    y=nan
end if
return y
end function

global function nPr(object a, object b)
atom y
y = factorial(a)/factorial(a-b)
return y
end function

global function nCr(object a,object b)
atom y
y = factorial(a)/(factorial(a-b)*factorial(b))
return y
end function

global function studentT95(integer n)
--t-statistic for normal distribution for alpha=0.025.
--Regression versus 1/n, t95(inf)=1.96.
atom x, x2, x3, x4, x5, t95
    x=1/n
    x2=x*x
    x3=x2*x
    x4=x3*x
    x5=x4*x
    t95=1.96+2.3728*x+2.8186*x2+2.6355*x3+1.0022*x4+1.917*x5
    return t95
end function

--Evaluate the determinate of a matrix.
global function determinate_eval(integer n, sequence a)
--a is a square matrix.
atom det, temp
sequence b
integer mult
    b=repeat(repeat(0,n-1),n-1)
    if n=2 then
	det=a[1][1]*a[2][2]-a[2][1]*a[1][2]
    else
	det=0
	mult=-1
	for i=1 to n do
	    mult*=-1
	    for k=1 to i-1 do
		for j=2 to n do
		    b[k][j-1]=a[k][j]
		end for
	    end for
	    for k=i+1 to n do
		for j=2 to n do
		    b[k-1][j-1]=a[k][j]
		end for
	    end for
	    temp=a[i][1]*determinate_eval(n-1,b)
	    temp=mult*temp
	    det+=temp
	end for
    end if
    return det
end function

puts(1,"math.e - copyright 2004 Donald Cahela\n")
