puts(1,"Test code for math.e-a library of mathematical functions that operate similarly to power(x,n)\n")
include math.e

--Comparison of execution time of series computation to evaluation of 
--exponential function by Euphoria power function is done in the test code.
--Computation using the power function is about 225 times faster than summing the 
--series expansion using a Dell Dimension 4100 PC.  Also, compare the execution
--time of AM's exp func to the minimalistic code used in math.e.  The check for
--input being a sequence adds about ten percent overhead.

function expseries_atom(atom x)
atom y, temp
integer count
    y=1
    temp=1
    count=0
    while (abs(temp)>1e-308) do
	count+=1
	temp*=x/count
	y+=temp
    end while
return y
end function


function exp_AM(object x)
object y
if sequence(x) then
    y = repeat(0,length(x))
    for count=1 to length(x) do
	y[count] = exp(x[count])
    end for
else
    y = power(2.71828182845904523536,x)
end if
return y
end function

--Calculation of square root by iteration. Comparison of execution time to 
--Euporia sqrt functon in test code.
function sqrt_iter(atom x)
atom y, ynew, err
integer count
    if x>0 then
	y=x/2
	err=1
	count=0
	while (err>1e-8) do
	    ynew=y+(x-y*y)/2/y
	    err=power((y-ynew),2)/x
	    y=ynew
	end while
    elsif (x=0) then
	y=0
    else
	y=nan
    end if
    return y
end function

include get.e
integer m

sequence x, y

x={1,2,3,{4,5}}
y=repeat(0,length(x))
atom var, stddev, time1, time2, time3, xtest, ytest, deta
integer n1, n2, n3
sequence an

puts(1,"x=")
?x
puts(1,"exp(x)=")
? exp(x)
puts(1,"sinh(x)=")
? sinh(x)
puts(1,"cosh(x)=")
? cosh(x)
puts(1,"tanh(x)=")
? tanh(x)

puts(1,"Hit any key to continue\n")
m=wait_key()
clear_screen()


puts(1,"\n\nTest code for exponential and variance\n")
printf(1,"e=expseries_atom(1)=%25.15g\n",expseries_atom(1))
printf(1,"x=log(2), expseries_atom(x)=%g, power(e,x)=%g\n",
    {expseries_atom(log(2)),power(2.71828182845904523536,log(2))})
xtest=5
time1=time()
n1=5000
printf(1,"Calculate exp(%d) by series expansion %d times\n",{xtest,n1})
for i=1 to n1 do
    ytest=expseries_atom(xtest)
end for
time1-=time()
time1=-time1
printf(1,"time to compute sum(x^i/n!),i=0,1,2,... for  x=%d, %d times=%g\n",{xtest,n1,time1})

time2=time()
n2=500000
for i=1 to n2 do
    ytest=exp_AM(xtest)
end for
time2-=time()
time2=-time2
printf(1,"time to compute power(e,%d) %d times using AM's func=%g\n",{xtest,n2,time2})
printf(1,"Ratio of execution time for exp series to exp func using power(e,x)=%g\n",((time1/n1)/(time2/n2)))

time3=time()
n3=500000
printf(1,"Calculate exp(%d) by power(e,x) %d times\n",{xtest,n2})
for i=1 to n3 do
    ytest=exp(xtest)
end for
time3-=time()
time3=-time3
printf(1,"time to compute power(e,%d) %d times=%g\n",{xtest,n3,time3})
printf(1,"Ratio of execution time for exp series to exp func using power(e,x)=%g\n",((time1/n1)/(time3/n3)))
printf(1,"AM's power(e,x) is %g %% slower than power(e,x)\n",((time2/n2)/(time3/n3)-1)*100)

--test code for variance and standard deviation functions.
puts(1,"\n\ntest variance computation on sequence of numbers\n")
puts(1,"Test sequence=")
x={1,2,3,4,5,6,7,8,9,10}
? x
var=variance(x)
stddev=std_dev(x)
puts(1,"Result computed by MSExcel is var=9.16666666... and std=3.02765...\n")
printf(1,"Computation by function in math.e: Variance=%g, Standard deviation=%g\n\n",{var,stddev})

xtest=5
puts(1,"\nTest iteration for sqrt versus Euphoria sqrt\n")
printf(1,"sqrt(%g)=%g",{xtest,sqrt(xtest)})
printf(1," sqrt_iter(%g)=%g\n",{xtest,sqrt_iter(xtest)})
xtest=1000
time1=time()

n1=100000
printf(1,"Calculate sqrt(%d) by iteration %d times\n",{xtest,n1})
for i=1 to n1 do
    ytest=sqrt_iter(xtest)
end for
time1-=time()
time1=-time1
printf(1,"time to compute sqrt by iteration for  x=%d, %d times=%g\n",{xtest,n1,time1})

time2=time()
n2=1000000
for i=1 to n2 do
    ytest=sqrt(xtest)
end for
time2-=time()
time2=-time2
printf(1,"time to compute sqrt(%d) %d times using Euphoria func=%g\n",{xtest,n2,time2})
printf(1,"Ratio of execution time for sqrt iter to sqrt func=%g\n",((time1/n1)/(time2/n2)))

time3=time()
n3=100000
printf(1,"Calculate sqrt(%d) as exp(log(x)/2) %d times\n",{xtest,n2})
for i=1 to n3 do
    ytest=exp(log(xtest)/2)
end for
time3-=time()
time3=-time3
printf(1,"time to compute exp(log(%d)/2) %d times=%g\n",{xtest,n3,time3})
printf(1,"Ratio of execution time for exp(log(x)/2) to sqrt func=%g\n",((time3/n3)/(time2/n2)))

puts(1,"Hit any key to continue\n")
m=wait_key()
clear_screen()

--  Test code for cmath.e
--x={1,1,"c"}
--puts(1,"\nc_sinh of")
--print(1,x)
--puts(1,"=")
--? c_sinh(x)

puts(1,"Error function\n")
for i=1 to 9 do
    printf(1,"x=%g, erf(x)=%g and erf(-x)=%g\n",{i/10,erf(i/10),erf(-i/10)})
end for
for i=9 to 1 by -1 do
    printf(1,"x=%g, erf(x)=%g, erfc(x)= %g,  and erf(-x)=%g, erfc(-x)=%g\n",{i,erf(i),erfc(i),erf(-i),erfc(-1)})
end for
for i=1 to 3 do
    printf(1,"x=%g, err(x)=%g, erfc(x)=%g\n",{xerrcut-0.15+i/10,erf(xerrcut-0.25+i/10),erfc(xerrcut-0.25+i/10)})
end for
for i=0 to 3 do
    printf(1,"x=%g, err(x)=%g, erfc(x)=%g\n",{xerrcut+i,erf(xerrcut+i),erfc(xerrcut+i)})
end for
puts(1,"Test values compared to values produced by gnumeric, OpenOfficeorg2.0(calc), and MSExcel\n")

--Test code for determinate of a square matrix
puts(1,"\nTest code for determinate evaluation\n")
an={{8,4,3,1},{4,3,1,2},{3,2,1,5},{5,8,3,2}}
deta=determinate_eval(4, an)
puts(1,"an=")
?an
puts(1,"determinate=")
?deta

puts(1,"End of test code for math.e - Don Cahela, 2004-2006\n")

