include math.e
clear_screen()
--Some procedures to manipulate polynomials
--global function poly_eval(atom x, sequence p)
--global function poly_mult(sequence pa, sequence pb)
--global function poly_determinate(integer n, sequence a)
--global function poly_derivative(sequence p)
--global function poly_roots_direct(atom xlow, atom xhigh, sequence p,
	--atom err, integer ndiv)

--Manipulation of polynomials
--poly={c0,c1,c2,...,cn}
--P=c0+c1*x+c2*x^2+c3*x^3+...cn*x^n
--Lenght of poly is n+1

global function poly_eval(atom x, sequence p)
atom y
    y=p[1]
    for i=2 to length(p) do
	y+=p[i]*power(x,i-1)
    end for
    return y
end function

--A function to multiply two polynomials
global function poly_mult(sequence pa, sequence pb)
sequence pc
    pc=repeat(0,(length(pa)+length(pb)-1))
    for i=1 to length(pa) do
	for j=1 to length(pb) do
	    pc[i+j-1]+=pa[i]*pb[j]
	end for
    end for
    return pc
end function

--Evaluate the determinate of a square matrix of polynomials as a polynomial.
global function poly_determinate(integer n, sequence a)
--a is a square matrix with each entry being a polynomial.
sequence polydet, b, temp
integer mult, l1, l2
    polydet=repeat(0,1)
    b=repeat(repeat(repeat(0,1),n-1),n-1)
    if n=2 then
	polydet=poly_mult(a[1][1],a[2][2])-poly_mult(a[2][1],a[1][2])
    else
	mult=-1
	for i=1 to n do
	    mult*=-1
	    for k=1 to i-1 do
		for j=2 to n do
		    b[k][j-1]=a[k][j]
		end for
	    end for
	    for k=i+1 to n do
		for j=2 to n do
		    b[k-1][j-1]=a[k][j]
		end for
	    end for
	    temp=poly_mult(a[i][1],poly_determinate(n-1,b))
	    temp=mult*temp
	    l1=length(temp)
	    l2=length(polydet)
	    if l1!=l2 then
		if l1>l2 then
		    polydet=polydet&repeat(0,l1-l2)
		end if
	    end if
	    for j=1 to l1 do
		polydet[j]+=temp[j]
	    end for
	end for
    end if
    return polydet
end function

--Take the derivative of a polynomial
global function poly_derivative(sequence p)
sequence pderv
    pderv=repeat(0,length(p)-1)
    for i=2 to length(p) do
	pderv[i-1]=(i-1)*p[i]
    end for
    return pderv
end function

--Newton's search on a polynomial to refine root.
function poly_newton_search(atom x, sequence p, sequence dpdx, atom err)
atom x1, d1, f1, f2
    f1=1
    while abs(f1)>err do
	f1=poly_eval(x,p)
	d1=poly_eval(x,dpdx)
	x1=x-f1/d1
	f2=poly_eval(x1,p)
	if abs(f2)<abs(f1) then
	    x=x1
	else
	    x=(x+x1)/2
	end if
    end while
    return x1
end function

--Find zero's of a polynomial. Input range to look in.
global function poly_roots_direct(atom xlow, atom xhigh, sequence p,
	atom err, integer ndiv)
--If sign of y changes then a root lies between end points.
sequence proots, dpdx
atom deltx, y1, y2, x1, x2
integer nroot, i
    proots=repeat(0,1)
    puts(1,"Finding zeros of a polynomial\n")
    deltx=(xhigh-xlow)/ndiv
    x1=xlow
    x2=x1
    i=0
    while x2<xhigh do
	x2=x1+deltx
	y1=poly_eval(x1,p)
	y2=poly_eval(x2,p)
--If y changes sign, or either value is zero, then a root lies between them
	if y1*y2<=0 then
	    i+=1
	    if length(proots)<i then
		proots=proots&repeat(0,1)
	    end if
	    if y1*y2<0 then
--Linear interpolation to improve root      
		proots[i]=x1+deltx*y1/(y2-y1)
	    else
		if y1=0 then
		    proots[i]=x1
		else
		    if y2=0 then
			proots[i]=x2
		    end if
		end if
	    end if
	end if
	if y2!=0 then
	    x1=x2
	else
	    x1=x2+deltx/ndiv
	end if
    end while
    nroot=length(proots)
    printf(1,"%d roots of polynomial found\n",nroot)
    dpdx=poly_derivative(p)
--Improve roots using newton iteration
    for j=1 to  nroot do
	proots[j]=poly_newton_search(proots[j], p, dpdx, err)
    end for
    return proots
end function

puts(1,"poly.e - copyright 2006 Donald Cahela\n")
