include math.e
clear_screen()
--  A routine to search for zeros of a polynomial is in poly.e.

--  Search for zeros of a general function
function func_num_derv(atom x, atom fx, atom deltx, integer func_id, sequence parms)
atom df
    df=(call_func(func_id,{x+deltx,parms})-fx)/deltx
    return df
end function

--Newton's search on a general function to refine root.
function newton_1D_search(atom x1, atom f1, integer func_id, sequence parms, 
    atom deltx, atom err, integer nmax)
atom x2, d1, f2
integer i
    i=0
    while abs(f1)>err and i<nmax do
	i+=1
	d1=func_num_derv(x1, f1, deltx, func_id, parms)
	if d1!=0 then
--  If derv is not zero then apply Newton's search formula          
	    x2=x1-f1/d1
	    f2=call_func(func_id,{x2,parms})
	    if abs(f2)<abs(f1) then
		x1=x2
		f1=f2
	    else
--  If f2 > f1 then Newton's formula stepped too far.
		x1=(x1+x2)/2
		f1=call_func(func_id,{x1,parms})
	    end if
	else
	    x1+=deltx
	end if
    end while
    return x1
end function

--Search for next zero of a general function. 
--Input starting value and step size to search for next root.
--Positive step size for search in increasing x values.
--Negative step size for search in decreasing x values.
global function funcx_nextroot(atom x1, atom xinc, 
    integer func_id, sequence parms, atom err)
--If sign of y changes then a root lies between points.
atom y1, y2, x2, xroot
integer i, rootfound
    --puts(1,"Finding next zero of a general function\n")
    i=0
    y1=call_func(func_id,{x1,parms})
--  Allow 1000 steps to find change in sign of function.
    rootfound=0
    while i<1000 and rootfound=0  do
	x2=x1+xinc
	y2=call_func(func_id,{x2,parms})
--If y changes sign, or either value is zero, then a root lies between them
	if y1*y2<=0 then
	    i+=1
	    if y1*y2<0 then
		rootfound=1
--Linear interpolation to improve root      
		xroot=x1+xinc*y1/(y2-y1)
	    else
		if y1=0 then
		    xroot=x1
		else
		    if y2=0 then
			xroot=x2
		    end if
		end if
	    end if
	end if
	if y2!=0 then
	    x1=x2
	else
	    x1=x2+xinc
	    y1=y2
	end if
    end while
--Improve root of general function using newton iteration.
    xroot=newton_1D_search(xroot, y1, func_id, parms, xinc/100, err, 100)
    return xroot
end function

--Search for next zero of a general function. 
--Input starting value and step size to search for next root.
--Positive step size for search in increasing x values.
--Negative step size for search in decreasing x values.
--reduce step size by one-half and continue search until xinc is less than the
--desired tolerance-interval halfing.
--Added 
global function funcx_nextroot_intvhalf(atom x1, atom xinc, atom xlimit,
    integer func_id, sequence parms, atom tol)
--If sign of y changes then a root lies between points.
atom y1, y2, x2, xroot, y1y2
integer i, ilimit
    ilimit=floor(log(abs(xinc/tol))/log(2)*abs((xlimit-x1)/xinc))
    --puts(1,"Finding next zero of a general function\n")
    i=0
    y1=call_func(func_id,{x1,parms})

    while i<ilimit and abs(xinc)>tol  do
	x2=x1+xinc
	y2=call_func(func_id,{x2,parms})
--If y changes sign, or either value is zero, then a root lies between x1 and x2
--reduce the interval by one-half and continue the search 
	y1y2=y1*y2
	if y1y2<=0 then
	    i+=1
	    if y1y2<0 then
		xinc/=2
--When the interval is less than the tolerance, refine the estimate of the root
--by linear interpolation.
		if abs(xinc) < tol then
		    xroot=x1+xinc*y1/(y2-y1)
		end if
	    elsif y1=0 then
		xroot=x1
		xinc=0
	    elsif y2=0 then
		xroot=x2
		xinc=0
	    end if
	else
	    x1=x2
	    y1=y2
	end if
    end while
    
    return xroot
end function


puts(1,"roots.e, Search for roots of a general function, copyright 2006 Donald Cahela\n")
