puts(1,"Test code for roots.e-Search for root of a general function\n")
include roots.e

--  Test root finder for general function of one variable.
--  A Butler-Volmer type function for examining distribution of current density
--  on an electrode due to consumption of the reactants.
function Butler_Volmer_test_func(atom volt, sequence parms)
atom I, I0, TK, beta, x1, x2, x3, area
atom RJK, Faradayconst, BFRTc
    area=50     --cm2
    RJK=8.31433     --Universal gas constant in Joules/gmmole-Kelvin
    Faradayconst= 96487.0   --Faraday's constant in coulomb/gmmole
    I=parms[1]
    I0=parms[2]
    TK=parms[3]+273.16
    beta=parms[4]
    x1=parms[5]
    x2=parms[6]
    x3=parms[7]
    BFRTc=beta*Faradayconst/RJK/TK
    return (I0*area/3*(x1*exp(-volt*BFRTc+beta/2*log(x1))
	+x2*exp(-volt*BFRTc+beta/2*log(x2))
	+x3*exp(-volt*BFRTc+beta/2*log(x3))) 
	-I)
end function

integer BVfunc_id, tf
atom xBV, I, I0, Tc, beta, xH2a, xH2b, xH2c, val
I0=20   --amp/cm2
Tc=25
beta=0.25
xH2a=5/6    --(1+2/3)/2
xH2b=1/2    --(2/3+1/3)/2
xH2c=1/6    --(1/3+0)/2

tf=open("roottest.txt","w")
puts(1,"Output of test code for funcx_nextroot\n")
puts(tf,"Output of test code for funcx_nextroot\n")
BVfunc_id=routine_id("Butler_Volmer_test_func")
puts(1,"Show values of function for 1 amp and various voltages\n")
puts(tf,"Show values of function for 1 amp and various voltages\n")
for i=1 to 10 do
    xBV=i/10
    val=call_func(BVfunc_id,{xBV,{1,I0,Tc,beta,xH2a,xH2b,xH2c}})
    printf(1,"E=%g, BVfunc=%g\n",{xBV,val})
    printf(tf,"E=%g, BVfunc=%g\n",{xBV,val})
end for
puts(1,"\n\nTesting Function to find root of a general function of one variable\n")
puts(1,"Parameters for Butler-Volmer test function\n")
printf(1,"I0=%g, TCentigrade=%g, beta=%g, xH2a=%g, xH2b=%g, xH2c=%g\n",{I0,Tc,beta,xH2a,xH2b,xH2c})
puts(1,"\n\nTest root finder for cell voltage as function of current\n")
puts(1,"Current, voltage\n")
puts(tf,"\n\nTest root finder-determine cell voltage as function of current\n")
puts(tf,"Current, voltage\n")
for i=1 to 20 do
    I=i*2
    xBV=funcx_nextroot(-2, 0.1, BVfunc_id, {I,I0,Tc,beta,xH2a,xH2b,xH2c}, 1e-10)
    printf(1,"%4g amp, %g volt\n",{I,xBV})
    printf(tf,"%4g amp, %g volt\n",{I,xBV})
end for

--  The Newton-Raphson search method can diverge.  Interval halving is a more
--  reliable method.
puts(1,"\nRepeat test using interval halving version\n")
puts(tf,"\nRepeat test using interval halving version\n")
--Interval halving version has an additional parameter.
puts(1,"Current, voltage\n")
puts(tf,"Current, voltage\n")
for i=1 to 20 do
    I=i*2
    xBV=funcx_nextroot_intvhalf(-2, 0.1, 10, BVfunc_id, {I,I0,Tc,beta,xH2a,xH2b,xH2c}, 1e-10)
    printf(1,"%4g amp, %g volt\n",{I,xBV})
    printf(tf,"%4g amp, %g volt\n",{I,xBV})
end for

close(tf)
puts(1,"End of test code for roots.e - Don Cahela, 2006\n")

