--Copyright (C) 2005 Donald Cahela

--  Dr. Donald R. Cahela
--  Center for Microfibrous Materials Manufacturing
--  M110 Wilmore Laboratories
--  Auburn University, Alabama 36849
--  email:  caheldr@auburn.edu

--These functions return estimates of statistical distributions using recursive
--integration by Simpson's rule or Boole's 5pt rule.
include math.e
--include simpsrec.e
include boolsrec.e
clear_screen()

atom tolintg
tolintg=1e-18

function gamma_integrand(atom t, sequence parms)
    return power(t,parms[1]-1)*exp(-t)
end function

function normal_dist_integrand(atom z, sequence parms)
    return exp(-power(z,2)/2)
end function

function student_dist_integrand(atom t, sequence parms)
    return 1/power((1+power(t,2)/parms[1]),(parms[1]+1)/2)
end function

function Fdist_integrand(atom x, sequence parms)
integer m, n
    m=parms[1]
    n=parms[2]
    return power(x,(m/2-1))/power((n+m*x),(m+n)/2)
end function

function ChiSqr_dist_integrand(atom t, sequence parms)
    return power(t,parms[1]/2-1)*exp(-t/2)
end function

integer gamma_id, student_id, norm_id, Fdist_id, ChiSqr_id
gamma_id=routine_id("gamma_integrand")
norm_id=routine_id("normal_dist_integrand")
student_id=routine_id("student_dist_integrand")
Fdist_id=routine_id("Fdist_integrand")
ChiSqr_id=routine_id("ChiSqr_dist_integrand")

global function gamma_func_calc1(atom z)
atom zlow, value, err, fact
    fact=10
    value=Booles_rule_recursive_calc(0.001,10,tolintg,gamma_id,{z})
--Integral of gamma func integrand from 0 to 0.001. 
    err=1
    zlow=0.001/fact
    while err>10*tolintg do
	err=Booles_rule_recursive_calc(zlow,fact*zlow,tolintg,gamma_id,{z})
	value+=err
	zlow/=fact
    end while
--Integral of gamma func integrand from 10 to infinity. 
    zlow=10
    err=1
    while err>10*tolintg do
	err=Booles_rule_recursive_calc(zlow,fact*zlow,tolintg,gamma_id,{z})
	value+=err
	zlow*=fact
    end while
    return value
end function

global function gamma_func_calc(atom z)
--Procedure only for z>0
--Utilize the recursive nature of the gamma function to speed up the computation.
atom value
    if z>2 then
	value=(z-1)*gamma_func_calc(z-1)
    else
	value=gamma_func_calc1(z)
    end if
    return value
end function

global function normal_dist_calc(atom z)
atom intg_br
    intg_br=Booles_rule_recursive_calc(0,z,tolintg,norm_id,{1})
    intg_br/=sqrt(2*PI)
    intg_br+=0.5
    return intg_br
end function

global function student_dist_calc(integer df, atom z)
atom sdistfactor, intg_br
    sdistfactor=gamma_func_calc((df+1)/2)
    sdistfactor/=gamma_func_calc(df/2)
    sdistfactor/=sqrt(df*PI)
    intg_br=Booles_rule_recursive_calc(0,z,tolintg,student_id,{df})
    intg_br*=sdistfactor
    intg_br+=0.5
    return intg_br
end function

global function Fdist_calc(integer m, integer n, atom F)
atom Fdistfactor, intg_br, xlow, errF
    Fdistfactor=gamma_func_calc((m+n)/2)
    Fdistfactor/=gamma_func_calc(m/2)
    Fdistfactor/=gamma_func_calc(n/2)
    Fdistfactor*=power(m,m/2)*power(n,n/2)
    if m>2 then
	intg_br=Booles_rule_recursive_calc(0,F,tolintg,Fdist_id,{m,n})
    else
	xlow=0.1
	intg_br=Booles_rule_recursive_calc(xlow,F,tolintg,Fdist_id,{m,n})
	errF=1
	while errF>10*tolintg do
	    xlow/=10
	    errF=Booles_rule_recursive_calc(xlow,10*xlow,tolintg,Fdist_id,{m,n})
	    intg_br+=errF
	end while
    end if
    intg_br*=Fdistfactor
    return intg_br
end function

global function Chi_Sqr_dist_calc(integer df, atom xsqr)
atom intg_br
    intg_br=Booles_rule_recursive_calc(0,xsqr,100*tolintg,ChiSqr_id,{df})
    intg_br/=gamma_func_calc(df/2)
    intg_br/=power(2,df/2)
    return intg_br
end function

function calc_dist_choice(integer dchoice, integer df, integer m, integer n, atom z)
atom F
    if dchoice=1 then
	F=normal_dist_calc(z)
    else
	if dchoice=2 then
	    F=student_dist_calc(df,z)
	else
	    if dchoice=3 then
		F=Fdist_calc(m,n,z)
	    else
		F=Chi_Sqr_dist_calc(df,z)
	    end if
	end if
    end if
    return F
end function

function zofdistforF(integer dchoice, integer df, integer m, integer n, atom F)
--Find value of z such that the tail of the distribution equals alpha.
--dchoice=  1: normal distribution
--          2: student-t distribution, one parameter-df
--          3: F-distribution, two parameters, m and n
--          4: Chi square ditribution, one parameter-df
atom z1, z2, znew, F1, F2, Fnew, tol, err1, err2, err
    tol=1e-4
    --fact=2
--Find the integer so that the Fdist value is greater than valF.
    z2=0
    F2=0
    while F2<F do
	z2+=1
	F2=calc_dist_choice(dchoice, df, m, n, z2)
    end while
    z1=z2-1
    
    F1=calc_dist_choice(dchoice, df, m, n, z1)
--Linear interpolation to improve the value
    znew=lin_intp(z1,z2,F1,F2,F)
    Fnew=calc_dist_choice(dchoice, df, m, n, znew)
    err=abs(Fnew-F)
    while err>tol do
	err1=abs(F1-F)
	err2=abs(F2-F)
	if err1<err2 then
	    z2=znew
	    F2=Fnew
	else
	    z1=znew
	    F1=Fnew
	end if
	znew=lin_intp(z1,z2,F1,F2,F)
	Fnew=calc_dist_choice(dchoice, df, m, n, znew)
	err=abs(Fnew-F)
    end while
    return znew
end function

--Calls to the z of distforalpha.  The inverse distribution function.
global function inverse_normal_dist(atom F)
    return zofdistforF(1, 0, 0, 0, F)
end function

global function inverse_student_tdist(integer df, atom F)
    return zofdistforF(2, df, 0, 0, F)
end function

global function inverse_Fdist(integer m, integer n, atom F)
    return zofdistforF(3, 0, m, n, F)
end function

global function inverse_Chi_Sqr_dist(integer df, atom F)
    return zofdistforF(4, df, 0, 0, F)
end function

puts(1,"StatFunc.e-copyright Donald Cahela 2006\n")
